/* ************************************************************************* */
/*                                                                           */
/*    dspoff.c: Run to trun off displays TS7400                              */
/*                                                                           */
/*    Copyright (C) 2010  Len Bayles, TMD Innovations                        */
/*                                                                           */
/*    This program is free software: you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation, either version 3 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program.  If not, see <http://www.gnu.org/licenses.    */
/*                                                                           */
/* ************************************************************************* */

#include <fcntl.h>
#include <stdio.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>


#define CLOCK                   0x10
#define DATA                    0x20
#define DISP0                   0x40
#define DISP1                   0x80


int	init_display(unsigned char *gpioregs);

int	dwrite(unsigned char *gpioregs, char *buffer);
int	bwrite(unsigned char *gpioregs, char byte);
int	latch(unsigned char *gpioregs, char device);

int	init_io(unsigned char **gpioregs);


int	main()
	{
	int		fd;
	unsigned char	*gpioregs;


	if((fd = init_io(&gpioregs)) == -1)
		return(-1);

	if(init_display(gpioregs))	/* Init display drivers */
		return(-1);


	return(0);
	}


int	init_display(unsigned char *gpioregs)
	{
	unsigned char	buffer[3];


	buffer[0] = DISP0;	/* Set Shutdown mode DISP0 */
	buffer[1] = 0x0c;
	buffer[2] = 0x00;

	dwrite(gpioregs, buffer);


	buffer[0] = DISP0;	/* Set Shutdown mode DISP0 */
	buffer[1] = 0x0c;
	buffer[2] = 0x00;

	dwrite(gpioregs, buffer);


	buffer[0] = DISP1;	/* Set Shutdown mode DISP1 */
	buffer[1] = 0x0c;
	buffer[2] = 0x00;

	dwrite(gpioregs, buffer);


	buffer[0] = DISP1;	/* Set Shutdown mode DISP1 */
	buffer[1] = 0x0c;
	buffer[2] = 0x00;

	dwrite(gpioregs, buffer);


	return(0);
	}


int	dwrite(unsigned char *gpioregs, char *buffer)
	{
	bwrite(gpioregs, buffer[1]);
	bwrite(gpioregs, buffer[2]);
	latch(gpioregs, buffer[0]);

	return(0);
	}


int     bwrite(unsigned char *gpioregs, char byte)
        {
        int     i;


        for(i = 8 ; i ; i--)
                {
                if(byte & 0x80)
                        gpioregs[2] |= DATA;
                else
                        gpioregs[2] &= ~DATA;

                gpioregs[2] |= CLOCK;
                gpioregs[2] &= !CLOCK;
                byte <<= 1;
                }

        return(0);
        }


int     latch(unsigned char *gpioregs, char device)
        {
        gpioregs[2] |= device;
        gpioregs[2] &= ~device;

        return(0);
        }


int	init_io(unsigned char **gpioregs)
	{
	int	fd;


	if((fd = open("/dev/mem", O_RDWR | O_SYNC)) < 0)
		return(-1);

	*gpioregs = (char *)mmap(0, 4096, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0x12c00000);

	(*gpioregs)[0] = 0xF0; /* dir reg 0 -> 3 inputs ; 4 -> 7 outputs */

	(*gpioregs)[2] = 0x0F;

	return(fd);
	}

